//-----------------------------------------------------------------------------
// main.c
//-----------------------------------------------------------------------------
//
// AUTH: FB
// DATE: 4 OCT 02
//
// Target: C8051F12x
// Tool chain: KEIL C51
//
// Description: 
// This is an example of how to set up and use the CMX Micronet HTTP 
// Web Server.
//
    
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <stdio.h>                     // for printf
#include "micronet.h"                  // for all CMX provided routines
#include "sfrdefs.h"                   // for all 8-bit and 16-bit sfr 
                                       // definitions not found in the standard
                                       // 8051 implementations. 

// include the header file for each HTML content file below
#include "HTML\index.h"                // header file for 'index.html'

//-----------------------------------------------------------------------------
// 16-bit SFR Definitions for 'F12x
//-----------------------------------------------------------------------------

sfr16 DP       = 0x82;                 // data pointer
sfr16 ADC0     = 0xbe;                 // ADC0 data
sfr16 ADC0GT   = 0xc4;                 // ADC0 greater than window
sfr16 ADC0LT   = 0xc6;                 // ADC0 less than window
sfr16 RCAP2    = 0xca;                 // Timer2 capture/reload
sfr16 RCAP3    = 0xca;                 // Timer3 capture/reload
sfr16 RCAP4    = 0xca;                 // Timer4 capture/reload
sfr16 TMR2     = 0xcc;                 // Timer2
sfr16 TMR3     = 0xcc;                 // Timer3
sfr16 TMR4     = 0xcc;                 // Timer4
sfr16 DAC0     = 0xd2;                 // DAC0 data
sfr16 DAC1     = 0xd2;                 // DAC1 data
sfr16 PCA0CP5  = 0xe1;                 // PCA0 Module 5 capture
sfr16 PCA0CP2  = 0xe9;                 // PCA0 Module 2 capture
sfr16 PCA0CP3  = 0xeb;                 // PCA0 Module 3 capture
sfr16 PCA0CP4  = 0xed;                 // PCA0 Module 4 capture
sfr16 PCA0     = 0xf9;                 // PCA0 counter
sfr16 PCA0CP0  = 0xfb;                 // PCA0 Module 0 capture
sfr16 PCA0CP1  = 0xfd;                 // PCA0 Module 1 capture

//-----------------------------------------------------------------------------
// Global CONSTANTS 
//-----------------------------------------------------------------------------
#define INTCLK       24500000          // Internal oscillator frequency in Hz
#define SYSCLK       49000000          // Output of PLL derived from (INTCLK*2)
#define BAUDRATE     9600              // Baud rate of UART in bps

sbit SW2 = P3^7;                       // SW2='0' means switch pressed
sbit LED = P1^6;                       // LED='1' means ON

//-----------------------------------------------------------------------------
// Global VARIABLES 
//-----------------------------------------------------------------------------
unsigned char code first_time = 0xFF;           // may be written once after 
                                                // each download because it is
                                                // initialized to 0xFF
char xdata* data ptr_first_time = &first_time; 


unsigned char code ip_address[4] _at_ 0x0000;   // located in Scratchpad area 
char xdata* data ptr_ip_address = &ip_address; 


unsigned char code mac_address[6] _at_ 0x0004;  // located in Scratchpad area
char xdata* data ptr_mac_address = &mac_address;

//-----------------------------------------------------------------------------
// Function PROTOTYPES
//-----------------------------------------------------------------------------
void main (void);
void SYSCLK_Init (void);
void PORT_Init (void);
void UART1_Init (void);
void EMIF_Init (void);
void ipconfig (void);

//-----------------------------------------------------------------------------
// MAIN Routine
//-----------------------------------------------------------------------------
void main (void) 
{

   WDTCN = 0xde;                       // Disable watchdog timer
   WDTCN = 0xad;
   
   // initialize the C8051F12x
   PORT_Init ();
   SYSCLK_Init (); 
   UART1_Init ();

   EMIF_Init ();
  
   // initialize the IP and MAC addresses and disable UART1
   ipconfig();
   
   // Set SFR page to LEGACY_PAGE
   SFRPAGE = LEGACY_PAGE;
      
   // initialize the CMX Micronet variables
   mn_init();
   
   // Add files to the Virtual File System.  Make sure you have allocated
   // enough slots in the Virtual File system for the number of files 
   // you add.  More slots can be allocated in 'mnconfig.h'
   mn_vf_set_entry((byte *)"index.html", 	INDEX_SIZE, index_html,	VF_PTYPE_FLASH);
   	
   // start the HTTP Server 
   mn_server();

   while(1);                           // This point in code should never 
                                       // be reached unless an error occurs

}

//-----------------------------------------------------------------------------
// Initialization Routines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// SYSCLK_Init
//-----------------------------------------------------------------------------
//
// This routine initializes the system clock to use the internal oscillator
// at 24.5 MHz multiplied by two using the PLL.
//
void SYSCLK_Init (void)
{
   int i;                           // software timer

   char SFRPAGE_SAVE = SFRPAGE;     // Save Current SFR page

   SFRPAGE = CONFIG_PAGE;           // set SFR page

   OSCICN = 0x83;                   // set internal oscillator to run
                                    // at its maximum frequency

   CLKSEL = 0x00;                   // Select the internal osc. as
                                    // the SYSCLK source
   
   //Turn on the PLL and increase the system clock by a factor of M/N = 2
   SFRPAGE = CONFIG_PAGE;
   
   PLL0CN  = 0x00;                  // Set internal osc. as PLL source
   SFRPAGE = LEGACY_PAGE;
   FLSCL   = 0x10;                  // Set FLASH read time for 50MHz clk
                                    // or less
   SFRPAGE = CONFIG_PAGE;
   PLL0CN |= 0x01;                  // Enable Power to PLL
   PLL0DIV = 0x01;                  // Set Pre-divide value to N (N = 1)
   PLL0FLT = 0x01;                  // Set the PLL filter register for 
                                    // a reference clock from 19 - 30 MHz
                                    // and an output clock from 45 - 80 MHz 
   PLL0MUL = 0x02;                  // Multiply SYSCLK by M (M = 2)
   
   for (i=0; i < 256; i++) ;        // Wait at least 5us
   PLL0CN  |= 0x02;                 // Enable the PLL
   while(!(PLL0CN & 0x10));         // Wait until PLL frequency is locked
   CLKSEL  = 0x02;                  // Select PLL as SYSCLK source

   SFRPAGE = SFRPAGE_SAVE;          // Restore SFR page
}


//-----------------------------------------------------------------------------
// PORT_Init
//-----------------------------------------------------------------------------
//
// Configure the Crossbar and GPIO ports
//
void PORT_Init (void)
{
   char SFRPAGE_SAVE = SFRPAGE;     // Save Current SFR page
   
   SFRPAGE = CONFIG_PAGE;           // set SFR page

   XBR0     = 0x00;                 
   XBR1     = 0x00;                 
   XBR2     = 0x44;                 // Enable crossbar, weak pull-ups,
                                    // and UART1

   P0MDOUT |= 0x01;                 // Set TX1 pin to push-pull
   P1MDOUT |= 0x40;                 // Set P1.6(LED) to push-pull
   
   // all pins used by the external memory interface are in push-pull mode 
   P4MDOUT =  0xFF; 
   P5MDOUT =  0xFF;
   P6MDOUT =  0xFF;
   P7MDOUT =  0xFF;                 
   P4 = 0xC0;                       // /WR, /RD, are high, RESET is low
   P5 = 0x00;  
   P6 = 0x00;                       // P5, P6 contain the address lines
   P7 = 0xFF;                       // P7 contains the data lines
   
   SFRPAGE = SFRPAGE_SAVE;          // Restore SFR page
   
}

//-----------------------------------------------------------------------------
// EMIF_Init
//-----------------------------------------------------------------------------
//
// Configure the External Memory Interface for Split-Mode to support both 
// on-chip and off-chip access.
//
void EMIF_Init (void){
   
   char SFRPAGE_SAVE = SFRPAGE;     // Save Current SFR page
   
   SFRPAGE = LEGACY_PAGE;
   
   EMI0CF = 0xF7;                   // Split-Mode, non-multiplexed
                                    // on P4 - P7

   EMI0TC = 0xB7;					      // This value may be modified 
                                    // according to SYSCLK to meet the 
                                    // timing requirements for the CS8900A
                                    // For example, EMI0TC should be >= 0xB7 
                                    // for a 100 MHz SYSCLK.
   SFRPAGE = SFRPAGE_SAVE;          // Restore SFR page
}

//-----------------------------------------------------------------------------
// UART1_Init
//-----------------------------------------------------------------------------
//
// Configure the UART1 using Timer1, for <baudrate> and 8-N-1.
//
void UART1_Init (void)
{
   char SFRPAGE_SAVE = SFRPAGE;     // Save Current SFR page
   
   SFRPAGE = UART1_PAGE;
   SCON1   = 0x10;                  // SCON1: mode 0, 8-bit UART, enable RX
   
   SFRPAGE = TIMER01_PAGE;
   TMOD   &= ~0xF0;
   TMOD   |=  0x20;                 // TMOD: timer 1, mode 2, 8-bit reload
   
   
   if (SYSCLK/BAUDRATE/2/256 < 1) {
      TH1 = -(SYSCLK/BAUDRATE/2);
      CKCON |= 0x10;                // T1M = 1; SCA1:0 = xx
   } else if (SYSCLK/BAUDRATE/2/256 < 4) {
      TH1 = -(SYSCLK/BAUDRATE/2/4);
      CKCON &= ~0x13;               // Clear all T1 related bits
      CKCON |=  0x01;               // T1M = 0; SCA1:0 = 01
   } else if (SYSCLK/BAUDRATE/2/256 < 12) {
      TH1 = -(SYSCLK/BAUDRATE/2/12);
      CKCON &= ~0x13;               // T1M = 0; SCA1:0 = 00
   } else {
      TH1 = -(SYSCLK/BAUDRATE/2/48);
      CKCON &= ~0x13;               // Clear all T1 related bits
      CKCON |=  0x02;               // T1M = 0; SCA1:0 = 10
   }   

   TL1 = TH1;                       // initialize Timer1
   TR1 = 1;                         // start Timer1  
   
   SFRPAGE = UART1_PAGE;
   TI1 = 1;                         // Indicate TX1 ready

   SFRPAGE = SFRPAGE_SAVE;          // Restore SFR page

}

//-----------------------------------------------------------------------------
// ipconfig
//-----------------------------------------------------------------------------
//
// Configure the IP address of the device through the serial port.
//
void ipconfig()
{
   char input_str[20];
   unsigned char data temp_char[6];
   char c;
   long i;
   bit ok_flag;
   bit EA_SAVE;                     // Preserve Current Interrupt Status
   char SFRPAGE_SAVE = SFRPAGE;     // Save Current SFR Page

   // prompt for the IP and MAC addresses if this is the first time the program
   // is run after a FLASH download or if the SW2(P1.6) button is pressed
   if (first_time || !SW2 ) { 
      
      SFRPAGE = UART1_PAGE;
       
      RI1 = 0;
      while (1) {
         
         printf("Press any key to continue\n");
         
         for ( i = 0; i < SYSCLK/100; i++) {
            if(RI1) break;
         }
         
         if (RI1) break;
         LED = ~LED;
      }
      RI1 = 0;

      ok_flag = 0;
      do { 
         SFRPAGE = UART1_PAGE;
         printf("\n\nEnter the IP address (eg.  10.10.10.163) >");
         gets(input_str, sizeof(input_str));
         sscanf(input_str, "%bu.%bu.%bu.%bu", &temp_char[0], &temp_char[1],
                                              &temp_char[2], &temp_char[3]);
      
         // check if IP address is entered correctly and write in FLASH
         printf("\n\nIs %bu.%bu.%bu.%bu correct?", temp_char[0], temp_char[1],
                                                   temp_char[2], temp_char[3]);
         c = getchar();
         if (c == 'y' || c == 'Y') {

               SFRPAGE = LEGACY_PAGE;

               // erase scratchpad area and write the ip address to FLASH
               EA_SAVE = EA;                       // preserve current interrupt state
               EA = 0;                             // disable interrupts
               FLSCL |= 0x01;                      // enable FLASH write/erase
               PSCTL  = 0x07;                      // MOVX erases scratchpad FLASH
      
               *ptr_ip_address = 0;                // initiate erase
           
               PSCTL  = 0x05;                      // MOVX writes scratchpad FLASH
            
               ptr_ip_address[0] = temp_char[0];   // write the first  byte
               ptr_ip_address[1] = temp_char[1];   // write the second byte
               ptr_ip_address[2] = temp_char[2];   // write the third  byte
               ptr_ip_address[3] = temp_char[3];   // write the fourth byte
            
            
               PSCTL  =  0x00;                     // MOVX writes target XRAM
               FLSCL &= ~0x01;                     // disable FLASH write/erase
               EA = EA_SAVE;                       // restore interrupts
               ok_flag = 1;
               
               SFRPAGE = UART1_PAGE;
               printf("\nIP address successfully programmed.\n");

         }   
      } while( !ok_flag);

      ok_flag = 0; 
      do { 
         
         SFRPAGE = UART1_PAGE;
         
         printf("\n\nEnter the MAC address (IA) (eg.  00-0B-3C-xx-yy-zz) >");
         gets(input_str, sizeof(input_str));
         sscanf(input_str, "%bX-%bX-%bX-%bX-%bX-%bX", &temp_char[0], &temp_char[1],
                                                      &temp_char[2], &temp_char[3], 
                                                      &temp_char[4], &temp_char[5]);
      
         // check if IP address is entered correctly and write in FLASH
         printf("\n\nIs %bX-%bX-%bX-%bX-%bX-%bX correct?", temp_char[0], temp_char[1],
                                                           temp_char[2], temp_char[3], 
                                                           temp_char[4], temp_char[5]);
         c = getchar();
         if(c == 'y' || c == 'Y'){

               SFRPAGE = LEGACY_PAGE;
               
               // write the MAC address to FLASH
               EA_SAVE = EA;                       // preserve current interrupt state
               EA = 0;                             // disable interrupts
               FLSCL |= 0x01;                      // enable FLASH write/erase
               PSCTL  = 0x05;                      // MOVX writes scratchpad FLASH
            
               ptr_mac_address[0] = temp_char[0];  // write the first  byte
               ptr_mac_address[1] = temp_char[1];  // write the second byte
               ptr_mac_address[2] = temp_char[2];  // write the third  byte
               ptr_mac_address[3] = temp_char[3];  // write the fourth byte
               ptr_mac_address[4] = temp_char[4];  // write the fifth  byte
               ptr_mac_address[5] = temp_char[5];  // write the sixth  byte
           
               PSCTL  = 0x01;                      // MOVX writes FLASH byte

               *ptr_first_time = 0x00;             // clear the first_time flag
                                                   // Note: this flag is not in
                                                   // the scratchpad area.            

               PSCTL  =  0x00;                     // MOVX writes target XRAM
               FLSCL &= ~0x01;                     // disable FLASH write/erase
               EA = EA_SAVE;                       // restore interrupts
               
               ok_flag = 1;
               
               SFRPAGE = UART1_PAGE;
               printf("\nMAC address successfully programmed.\n");
         } 
      } while( !ok_flag);
   
   }
   
   // Disable Timer1
   SFRPAGE = TIMER01_PAGE;
   TR1 = 0;                                        // Stop Timer1
   TMOD = 0x00;                                    // Restore the TMOD register to 
                                                   // its reset value
   CKCON = 0x00;                                   // Restore the CKCON register to
                                                   // its reset value

   // Disable UART1                                             
   SFRPAGE = UART1_PAGE;
   SCON1 = 0x00;                                   // Disable UART1


   SFRPAGE = LEGACY_PAGE;
   
   // Copy the IP and MAC address from the scratchpad area to the CMX variables
   // located in RAM.
   EA_SAVE = EA;                                   // preserve current interrupt state
   EA = 0;                                         // disable interrupts
   PSCTL = 0x04;                                   // enable reads from the scratchpad

   // read the IP and MAC address from FLASH into their appropriate arrays in memory
   ip_src_addr[0] = ip_address[0];
   ip_src_addr[1] = ip_address[1];
   ip_src_addr[2] = ip_address[2];
   ip_src_addr[3] = ip_address[3];
   eth_src_hw_addr[0] = mac_address[0];
   eth_src_hw_addr[1] = mac_address[1];
   eth_src_hw_addr[2] = mac_address[2];
   eth_src_hw_addr[3] = mac_address[3];
   eth_src_hw_addr[4] = mac_address[4];
   eth_src_hw_addr[5] = mac_address[5];
   
   PSCTL = 0x00;                                   // disable reads from the scratchpad
   EA = EA_SAVE;                                   // restore interrupts
                  
   SFRPAGE = SFRPAGE_SAVE;                         // Restore SFR page

}